/*****************************************************************/
/*      mynetsl.c                                                */
/*      Version 2.0                      xin zuo                 */
/*---------------------------------------------------------------*/
/*  mynetsl()                                                    */
/*****************************************************************/

/*
 * mynetsl() : vararg C Interface to the NetSolve Client
 */

#include <stdarg.h>
#include "core.h"
#include "client.h"
#include "mnemonics.h"
#include "callingsequence.h"
#include "cfortranclient.h"
#include "sequences.h"
#include "netsolve.h"
#include "mynetsl.h"
#include "client2proxy.h"
#include "client2server.h"
#include "proxyjobdesc.h"
#include "proxy.h"
#include <signal.h>
#include <time.h>
#include <stdio.h>
#include <sys/types.h>
#include <errno.h>


#include <strings.h>
#include <sys/ioctl.h>

#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>

#define HOSTNAME_LENGTH 256

int mynetsl(char *nickname,...)
{
  va_list argptr;
  char * buf;

  /* Getting the problem's nickname */
  va_start(argptr,nickname); 
  buf = strdup(nickname);

  return mynetslX(buf, argptr, NS_CALL_FROM_C, NS_BLOCK, NS_NOASSIGNMENT);
}



int mynetslX (char *buf, va_list arglist,int language, int blocking, int assignment)
{
  return mynetslX_common (buf, 1, arglist, NULL, language, blocking, assignment);
}



int mynetslX_common (char *pname, int mode_valist, va_list arglist,
                   void **arglist_arr, int language, int blocking,
                   int assignment)
{
  int i, len;
  int request_id;
  int status;
  void **tmp_output_ptr;
  int elapsed;
  NS_Node *act_node = 0;
  NS_ProblemDesc *pd;
  char *nickname, *serverhostname;
  char *buf;

  if(!pname) {
    ns_errno = NetSolveBadProblemName;
    return ns_errno;
  }

  buf = strdup(pname);

#if defined(DEBUG)
  ns_printinfo();
  fprintf(stderr, "Processing a Client Request: language = %d, "
                  "blocking = %d, assignment = %d\n", language,
          blocking, assignment);
#endif
    printf("%s", "finished.00\n");
  /* find a slot for the request */
  request_id = 0;
  while(requests[request_id] != NULL)
  {
    request_id++;
    if (request_id == NB_MAX_REQUESTS)
    {
      ns_errno = NetSolveTooManyPendingRequests;
      free(buf);
      return ns_errno;
    }
  }
 
  /*== proxy needs to be started ==*/
  if(!proxy_pid){
    if(netslinit(NULL) < 0){
      free(buf);
      return ns_errno;
    }
  }

#if defined(DEBUG)
  ns_printinfo();
  fprintf(stderr, "Setting Default Client Major ...\n");
#endif
  /*== Initializing the major ==*/
  if(language == NS_CALL_FROM_C)
    setMajorDefault("Row");
  else
    setMajorDefault("Col");

#if defined(DEBUG)
  ns_printinfo();
  fprintf(stderr, "Stripping `(` and `)` from problemname %s ...\n",
          buf);
#endif
  /*== Stripping off the parenthesis ==*/
  len = strlen(buf);
  if ((buf[len-2] != '(' ) ||
      (buf[len-1] != ')' ) )
  {
#if defined(DEBUG)
  ns_printinfo();
  fprintf(stderr, "There is a bad problem name...\n"); fflush(stderr);
#endif
    free(buf);
    ns_errno = NetSolveBadProblemName;
    return ns_errno;
  }
  buf[len-2] = '\0';

#if defined(DEBUG)
  ns_printinfo();
  fprintf(stderr, "Extracting the problem name [and serverhostname] ...\n");
  fflush(stderr);
#endif

  if(assignment)
  {
    /* Extracting the nickname, serverhostname */
    serverhostname = strtok(buf,":");
    nickname = strtok(NULL,"\0");

    if ((serverhostname == NULL)||
        (nickname == NULL))
    {
      free(buf);
      return NetSolveBadProblemName;
    }
  }
  else{
    nickname = buf;
    serverhostname = NULL;
  }


  /*== Getting the problem descriptor ==*/
  status = netsolveInfo(nickname, &pd);
  if (status == -1) {
    free(buf);
    return ns_errno;
  }


  // test
  /* general variables */
  int n;

  /* variables for blocking calls */
  //int info;
  double alpha, beta;
  double *mat;
  double *vec;

  n = 100;
  srand( time( NULL ) );

    /* --blocking calls example--------- */
    /* creating matrix & vector */
    mat = calloc(n*n, sizeof(double));
    for (i=0; i<n*n; i++)
      mat[i] = (double) (10.0*rand()/RAND_MAX);
    vec = calloc(n, sizeof(double));
    for (i=0; i<n; i++)
      vec[i] = (double) (10.0*rand()/RAND_MAX);
    alpha = 1.0;
    beta = 0.0;

    /* blocking call to NetSolve test netsl in mynetsl: done */
    // info = netsl("matmul()",mat,vec,n);

    /* error handling for blocking call */
//     if (info <0) {
//       netslerr(info);
//       exit(0);
//     }

    /* write message to stdout */
    printf("%s", "finished\n");

  // test end



  //printf("test");

  /* gets the server name of an appropriate server for the given problem */
  /* make it assignment */
  NS_RequestDesc *rd_getServer;
  rd_getServer = CP_sendJobRequest(pd,0,0,-666);
  serverhostname = strtok(rd_getServer->hostname, "\0");
  //fprintf(stdout, rd_getServer->IPaddr);
  //free(rd_getServer);
  assignment = 1;


#if defined(DEBUG)
  ns_printinfo();
  fprintf(stderr, "Problem name: %s\n", nickname);
  if(assignment){
    ns_printinfo();
    fprintf(stderr, "Assigned Server: %s\n", serverhostname);
  }
#endif

#if defined(WIN32)
  /*== Initialize WinSock ==*/
  if ((!WinSockInit()))
  {
    ns_errno = NetSolveNetworkError;
    free(buf);
    return ns_errno;
  }
#endif /* WIN32 */

#if defined(DEBUG)
  ns_printinfo();
  fprintf (stderr, "Getting Problem Descriptor via netsolveInfo()\n");
#endif




#if defined(DEBUG)
  ns_printinfo();
  fprintf (stderr, "Initializing NetSolve objects: %d in input and %d"
                    "in output\n",
                    pd->nb_input_objects, pd->nb_output_objects);
#endif

  /*== Initializing the objects and set Major==*/
  for (i=0;i<pd->nb_input_objects;i++)
  {
    int object_type,data_type;
    object_type = pd->input_objects[i]->object_type;
    data_type = pd->input_objects[i]->data_type;
    freeObject(pd->input_objects[i]);
    pd->input_objects[i] = createEmptyObject(object_type,data_type);

    if (pd->input_objects[i]->object_type == NETSOLVE_MATRIX)
      pd->input_objects[i]->attributes.matrix_attributes.major = my_major;
    else if(pd->input_objects[i]->object_type == NETSOLVE_SPARSEMATRIX)
      pd->input_objects[i]->attributes.sparsematrix_attributes.major = my_major;
    else
      continue;
  }
  for (i=0;i<pd->nb_output_objects;i++)
  {
    int object_type,data_type;
    object_type = pd->output_objects[i]->object_type;
    data_type = pd->output_objects[i]->data_type;
    freeObject(pd->output_objects[i]);
    pd->output_objects[i] = createEmptyObject(object_type,data_type);

    if (pd->output_objects[i]->object_type == NETSOLVE_MATRIX)
      pd->output_objects[i]->attributes.matrix_attributes.major = my_major;
    else if(pd->output_objects[i]->object_type == NETSOLVE_SPARSEMATRIX)
      pd->output_objects[i]->attributes.sparsematrix_attributes.major = my_major;
    else 
      continue;
  }

#if defined(DEBUG)
  ns_printinfo();
  fprintf (stderr, "Parsing the call stack... call StackToObjects()\n");
#endif

  /*== Parsing the stack ==*/
  if(mode_valist) {
    if (StackToObjects(language, pd, arglist,
                     pd->input_objects, pd->output_objects) == -1)
    {
      freeProblemDesc(pd);
      free(buf);
      return ns_errno;
    }
  }
  else {
    if (ptrArrayToObjects(language, pd, 
                     pd->input_objects, pd->output_objects, arglist_arr) == -1)
    {
      freeProblemDesc(pd);
      free(buf);
      return ns_errno;
    }
  }

#if defined(DEBUG)
  ns_printinfo();
  fprintf (stderr, "Assigning the Constant Values ... call "
                   "assignConstantValues()\n");
#endif

  /*== Processing the constants ==*/
  status = assignConstantValues(pd,pd->input_objects,pd->output_objects);
  if (status == -1)
  {
    freeProblemDesc(pd);
    free(buf);
    return ns_errno;
  }

#if defined(DEBUG)
  ns_printinfo();
  fprintf (stderr, "Assigning the Formula Values ... call "
                   "assignFormulaValues()\n");
#endif

  /*== Processing the formulae ==*/
  status = assignFormulaValues(pd,pd->input_objects,pd->output_objects);
  if (status == -1)
  {
    freeProblemDesc(pd);
    free(buf);
    return ns_errno;
  }

#if defined(DEBUG)
  ns_printinfo();
  fprintf (stderr, "Special Preprocessing for packed files ... \n");
#endif

  /*== Special preprocessing for the packed file type ==*/
  for (i=0;i<pd->nb_input_objects;i++)
  {
    if (pd->input_objects[i]->object_type == NETSOLVE_PACKEDFILES){
      pd->input_objects[i]->attributes.packedfiles_attributes.filenames =
      copyCharStarList(
        pd->input_objects[i]->attributes.packedfiles_attributes.filenames,
        pd->input_objects[i]->attributes.packedfiles_attributes.m);
    }
  }

  for (i=0;i<pd->nb_output_objects;i++)
  {
    if (pd->output_objects[i]->object_type == NETSOLVE_PACKEDFILES)
    {
      pd->output_objects[i]->attributes.packedfiles_attributes.defaultprefix =
        (char *)(pd->output_objects[i]->attributes.packedfiles_attributes.filenames);
      pd->output_objects[i]->attributes.packedfiles_attributes.filenames = NULL;
    }
  }

#if defined(DEBUG)
  ns_printinfo();
  fprintf (stderr, "Saving the Output pointers for strings ... \n");
#endif

  /*== Saving the tmp_output_ptr ==*/
  tmp_output_ptr = (void **)calloc(pd->nb_output_objects,sizeof(void*));
  for (i=0;i<pd->nb_output_objects;i++)
  {
    if (pd->output_objects[i]->object_type == NETSOLVE_STRING)
      tmp_output_ptr[i] = 
        pd->output_objects[i]->attributes.string_attributes.ptr;
  }

  if(netsl_sequence_status() == FALSE)  {
#if defined(DEBUG)
  ns_printinfo();
  fprintf (stderr, "Finally, about to submit the request ... \n");
#endif
    status = mysubmit_problem(blocking, assignment, serverhostname, pd,
                            pd->input_objects, pd->output_objects, &elapsed, request_id,
                            rd_getServer);

    if(blocking){
#if defined(DEBUG)
      ns_printinfo();
      fprintf(stderr, "Blocking Call Returned, copying string outputs "
                      "...\n");
#endif
      if (status != -1) {
        /*== take care of the strings in output ==*/
        for (i=0; i<pd->nb_output_objects; i++) {
          if (pd->output_objects[i]->object_type != NETSOLVE_STRING)
            continue;
          strcpy(tmp_output_ptr[i],
              pd->output_objects[i]->attributes.string_attributes.ptr);
          free(pd->output_objects[i]->attributes.string_attributes.ptr);
        }
        ns_errno = NetSolveOK;
      }

      free(tmp_output_ptr);
      freeProblemDesc(pd);
      free(buf);
      return ns_errno;
    }
    else{   /*== NOBLOCK ==*/
#if defined(DEBUG)
      ns_printinfo();
      fprintf(stderr, "Non-Blocking Call Returned, saving Request "
                      "Descriptor ...\n");
#endif
      if (status < 0) {
        free(tmp_output_ptr);
        freeProblemDesc(pd);
        free(buf);
        return ns_errno;
      }

      output_ptrs[request_id] = tmp_output_ptr;
      free(buf);
      return request_id;
    }
  }
  else {
#if defined(DEBUG)
      ns_printinfo();
      fprintf(stderr, "In Sequencing mode, not yet scheduling request\n");
#endif
    /*== if neccessary generate a NS_Node and insert into structure ==*/
    act_node = netsl_new_Node_Dep(pd, pd->nb_input_objects, pd->nb_output_objects);
    netsl_insert_Node (act_node, netsl_get_Sequence());

    /*== Analyzing dependencies if sequencing mode ==*/
    if (netsl_analyze_Graph(act_node,*netsl_get_Sequence())!=TRUE){
      fprintf(stderr,"Failed to analyze dependencies, giving up\n");
      free(buf);
      return ns_errno;
    }

    act_node->tmp_output_ptr = tmp_output_ptr;   /* store the string pointers */

    /*== do nothing, until netsl_sequence_end is reached ==*/
    free(buf);
    return ns_errno;
  }

}



/*
 * submit_problem, formerly netsolveBlocking()
 *
 * This function allows the user to perform either a synchronous
 * or asychronous computation.
 * It returns either an error code, or with the result in the user's
 * workspace. This function is build on top of two other functions
 * that are also part of the client interfaceimplemented in this
 * directory.
 */
int mysubmit_problem(int blocking, int assignment, char *
                   serverhostname, NS_ProblemDesc *pd,
                   NS_Object **input, NS_Object **output, int *elapsed, int request_id,
                   NS_RequestDesc *rd_getServer)
{
  int status;
  NS_RequestDesc *rd;

  /***********************************/
  /* Sending an asynchronous request */
  /***********************************/
  rd = mynetsolveSendRequest(assignment, serverhostname, pd,input,output, rd_getServer);
      printf("%s", "finished.1\n");
  if(rd == NULL){
    netsolvePerror("netsolveSendRequest");
    return -1;
  }

  /*****************************************/
  /* If Blocking, Waiting for that request */
  /*****************************************/
  if(blocking == NS_BLOCK){
    status = netsolveWaitRequest(assignment, &rd,elapsed);
    free(rd);
    return status;
  }
  else{
    rd->assignment = assignment;
    requests[request_id] = rd;
    return NetSolveOK;
  }
}




/*
 * netsolveSendRequest()
 *
 * This function sends an asynchronous request to NetSolve.
 */
NS_RequestDesc *
mynetsolveSendRequest(int assignment, char * serverhostname,
                    NS_ProblemDesc *pd, NS_Object **input,
                    NS_Object **output, NS_RequestDesc *rd_getServer)
{
  int status;
  int input_size, output_size, problem_size;
  NS_RequestDesc *rd;

      printf("%s", "finished.2\n");
printf("%d", assignment);

  if(assignment){
    while(1){

        printf("%s", "finished.3\n");
      /* YC */
#if defined YC
      fprintf(stderr,"netsolveSendRequest : assignment\n") ;
#endif
      /* FYC */
      




      // client test
   int sockfd, portno, n;
    struct sockaddr_in serv_addr;
    struct hostent *server;

    char buffer[256];

    portno = 6235;
    sockfd = socket(AF_INET, SOCK_STREAM, 0);
    if (sockfd < 0) 
        error("ERROR opening socket");
    server = gethostbyname("localhost");
    if (server == NULL) {
        fprintf(stderr,"ERROR, no such host\n");
        exit(0);
    }
    bzero((char *) &serv_addr, sizeof(serv_addr));
    serv_addr.sin_family = AF_INET;
    bcopy((char *)server->h_addr, 
         (char *)&serv_addr.sin_addr.s_addr,
         server->h_length);
    serv_addr.sin_port = htons(portno);
    if (connect(sockfd,&serv_addr,sizeof(serv_addr)) < 0) 
        error("ERROR connecting");
    printf("Please enter the message: ");
    bzero(buffer,256);
    fgets(buffer,255,stdin);
    n = write(sockfd,buffer,strlen(buffer));
    if (n < 0) 
         error("ERROR writing to socket");
    bzero(buffer,256);
    n = read(sockfd,buffer,255);
    if (n < 0) 
         error("ERROR reading from socket");
    printf("%s\n",buffer);



    NS_Communicator *comm;
    comm = initTransaction(sockfd,rd_getServer->encoding);
    //comm = (NS_Communicator *)calloc(1,sizeof(NS_Communicator));
    //comm->sock = sockfd;
    int tag = 518;
    //printf("%d", &tag);
    if (sendInt(comm, &tag) == -1)
      {
    endTransaction(comm);
          printf("%s", "finished3333\n");
    return -1;
      }
 
    if (recvInt(comm, &tag) == -1)
      {
    endTransaction(comm);
          printf("%s", "finished3333\n");
    return -1;
      }
    //printf("%d", tag);


    printf("%s", "comm done\n");
    //return 1;






      rd = newRequestDesc();
            //rd = CP_sendJobAssignment(pd, getIPaddr(serverhostname));
      rd->pd = pd;
      rd->hostname = serverhostname;
      rd->IPaddr = rd_getServer->IPaddr;
      rd->port = 6234;
      rd->encoding = rd_getServer->encoding;
        printf("%s", "finished.3333\n");


      //CP_sendJobAssignment(pd, getIPaddr(serverhostname));
      if (rd == NULL)
        return NULL;
      /* Add the local information */
      rd->input = input;
      rd->output = output;

           printf("%s", pd->name);
  if(sendProblemDesc(comm,pd) == -1){
    netsolvePerror("");
    endTransaction(comm);
    return;
   }


  int i;
      // comm send object
  /* Sending the objects */
  for (i=0;i<pd->nb_input_objects;i++)
  {
  if (pd->input_objects[i]->pass_back == FALSE) continue;

    if (sendObject(comm,rd->input[i]) == -1)
    {
      endTransaction(comm);
      return -1;
    }
  }







  
      // test
          printf("%s", "finished2\n");
          return -1;
      // this test done

      /* Send the input */
      if (CS_mysendInput(rd) == -1) {
        if (ns_errno == NetSolveFileError) {
          free(rd);
          return NULL;
        }
          printf("%s", "finished3\n");
        reportFailure(rd->IPaddr,rd->pd->nickname,
                      rd->request_id,HOST_ERROR);
      }
      else{
        break;
      }
    }
  }
  else{   /*== No Assignment ==*/


        printf("%s", "finished.4\n");
    /*== determine the input/output/problem sizes ==*/
    status = determineSizes(pd,input,output,
             &input_size,&output_size,&problem_size);
    if (status == -1)
      return NULL;

    /*== Send the request to the system ==*/
    while(1)
    {
      /* YC */
#if defined YC
      fprintf(stderr,"netsolveSendRequest : noassignment\n") ;
#endif
      /* FYC */

      rd = CP_sendJobRequest(pd,input_size,output_size,problem_size);
      if (rd == NULL)
        return NULL;
      /* Add the local information */
      rd->input = input;
      rd->output = output;
  
#if defined (VIEW)
      /* YC */
     fprintf(STDERR "Sending Input of Task %lu to Server %s\n",
	     rd->global_id,
	     rd->hostname ? rd->hostname : "Unknown");
     /* FYC */
#endif
     /* YC */
     /* The global ID is now in the rd, but in fact
     ** the client must not send its task global id to the server
     ** the proxy does this */
     /*-> REALLY ??? <-*/
     /* FYC */
      if (CS_sendInput(rd) == -1) {
        if (ns_errno == NetSolveFileError) {
          free(rd);
          return NULL;
        }
        if (ns_errno == NetSolveAuthenticationError){
          free(rd);
          return NULL;
        }

        reportFailure(rd->IPaddr,rd->pd->nickname,
                      rd->request_id,HOST_ERROR);
        free(rd);
      }
      else
        break;
    }
  }
  
  ns_errno = NetSolveOK;
  return rd;
}




/*
 *  CS_mysendInput()
 *
 */
int CS_mysendInput(NS_RequestDesc *rd)
{
  NS_Communicator *comm;
  int tag;
  int i;

  /********************************************/
  /* contact the server			      */
  /********************************************/
  comm = mycontactHost(NULL,rd->IPaddr,rd->port,rd->encoding);
  //return 1;
  if (comm == NULL)
  {
    netsolvePerror("contactHost()");
          printf("%s", "finished2222\n");
    return -1;
  } 

  if (recvInt(comm, &tag) == -1)
  {
    endTransaction(comm);
          printf("%s", "finished3333\n");
    return -1;
  }

#ifdef KERBEROS5
  if (tag == NS_PROT_KRB5_AUTH_REQUIRED)
  {
    if (SendKerberos5Credentials (rd->hostname, comm) < 0)
    {
      ns_errno = NetSolveAuthenticationError;
      endTransaction (comm);
      return -1;
    }
    if (recvInt (comm, &tag) == -1)
    {
      endTransaction (comm);
      return -1;
    }
  }
  /*
   * check for authentication-specific error codes here
   * before falling through to the problem-specific error codes
   */
  if (tag == NS_PROT_AUTH_FAILED)
  {
    ns_errno = NetSolveAuthenticationError;
    endTransaction (comm);
    return -1;
  }
#endif

  if (tag == NS_PROT_KRB5_AUTH_REQUIRED)
  {
    ns_errno = NetSolveAuthenticationError;
    endTransaction (comm);
          printf("%s", "finished21\n");
    return -1;
  }
  else if(tag != NS_PROT_ACCEPTED){
    ns_errno = NetSolveNotAllowed;
    endTransaction(comm);
          printf("%s", "finished22\n");
    return -1;
  }

  /* Sending the request ID */
  if ((sendInt(comm,rd->request_id) == -1) ||
  /* YC */
  /* in case of change, it is here that the client must send
  ** its task global id */
  /* FYC */
      (sendInt(comm,my_major) == -1))
  {
    endTransaction(comm);
    return -1;
  }


  /* Sending the objects */
  for (i=0;i<rd->pd->nb_input_objects;i++)
  {
  if (rd->pd->input_objects[i]->pass_back == FALSE) continue;

    if (sendObject(comm,rd->input[i]) == -1)
    {
      endTransaction(comm);
      return -1;
    }
  }
  endTransaction(comm);
  return 1;
}




/*
 * mycontactHost()
 *
 * This is a high level wrapper
 */
NS_Communicator *mycontactHost(char *hostname, NS_IPaddr_type IPaddr, int port,
            int encoding)
{
  NS_Socket_type sock;
  NS_Communicator *comm;

  sock = myconnectToSocket(hostname,IPaddr,port);
  if (isSocketError(sock))
    return NULL;
        printf("%s", "mycontactHost.1\n");
  comm = initTransaction(sock,encoding);
  if (comm == NULL)
  {
    closeSocket(sock);
    return NULL;
  }
  return comm;
}



NS_Socket_type myconnectToSocket(char *hostname,NS_IPaddr_type IPaddr,int port)
{

          printf("%s", "here\n");
  NS_Socket_type sock;
  struct sockaddr_in inet_addr;
  struct hostent *hp;
  NS_IPaddr_type tmpIPaddr;
  static NS_IPaddr_type myIPaddr=(NS_IPaddr_type)0;
  char *hn = 0;

  tmpIPaddr = IPaddr;

  if ((sock = socket(AF_INET,SOCK_STREAM,0)) == -1)
  {
    ns_errno = NetSolveSystemError;
    return -1;
  }
  BZERO((void*)&inet_addr,sizeof(inet_addr));

  /* if NULL,0 : this host, otherwise distant host */
  if ((hostname == NULL)&&(IPaddr == 0))
  {
    if (myIPaddr == (NS_IPaddr_type)0)
      myIPaddr = getMyIPaddr();
    tmpIPaddr = myIPaddr;
  }
  else if (IPaddr != 0)
  {
    tmpIPaddr = IPaddr;
  }
  else if (hostname != NULL)
  {
    hn = hostname;
  }

  if (tmpIPaddr != (NS_IPaddr_type)0)
    hp = gethostbyname(ns_ipaddr_ntoa(tmpIPaddr));
  else 
    hp = NULL;

  if (hp == NULL)
  {
    /* Try to do a gethostbyname() */
    if (hn != NULL)
      hp = gethostbyname(hn);
    else
      hp = NULL;
    if (hp == NULL)
    {
      close(sock);
      ns_errno = NetSolveUnknownHost;
      return -1;
    }
  }
  BCOPY((void*)hp->h_addr,(void*)&inet_addr.sin_addr,hp->h_length);
  inet_addr.sin_family = AF_INET;
  inet_addr.sin_port = htons(port);
  
#ifdef DEBUG
  if (hn != NULL)
    fprintf(STDERR"Connecting to %s ...",hn);
#endif

          printf("%s", "here1\n");
  if (connect(sock,(struct sockaddr *)&inet_addr,sizeof(inet_addr)) == -1)
  {
#ifdef DEBUG
    fprintf(STDERR"failure\n",hn);
#endif
    close(sock);
    ns_errno = NetSolveConnectionRefused;
          printf("%s", "here2\n");
    return -1;
  }
#ifdef DEBUG
  fprintf(STDERR"success\n",hn);
          printf("%s", "here3\n");
#endif
  return sock;
}
