/****************************************************************/
/*      server.c                                                 */
/*      Henri Casanova, Dorian Arnold                            */
/*---------------------------------------------------------------*/
/*  main()                                                       */
/*****************************************************************/

#include "core.h"
#include <signal.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include "serverglobal.h"
#include "serverinit.h"
#include "serverprocessmessage.h"
#include "netsolvesignals.h"
#include "serverConnector.h"
#include <stdio.h>
#include <sys/types.h>
#include <errno.h>

#ifndef WIN32
#include <strings.h>
#include <sys/ioctl.h>
#if (defined(solaris))
#include <sys/filio.h>
#endif
#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>
#endif /* UNIX */

#ifdef WIN32
#include "win_util.h"
#endif /* WIN32 */



#define HOSTNAME_LENGTH 256




/*
 * Main routine. Does nothing but calling the initialization routine
 * and waiting for a network connection
 */


int main(int argc,char **argv)
{

  // test
  /* general variables */
  int n,i;

  /* variables for blocking calls */
  //int info;
  double alpha, beta;
  double *mat;
  double *vec;

  n = 100;
  int info;
  srand( time( NULL ) );

    /* --blocking calls example--------- */
    /* creating matrix & vector */
    mat = calloc(n*n, sizeof(double));
    for (i=0; i<n*n; i++)
      mat[i] = (double) (10.0*rand()/RAND_MAX);
    vec = calloc(n, sizeof(double));
    for (i=0; i<n; i++)
      vec[i] = (double) (10.0*rand()/RAND_MAX);
    alpha = 1.0;
    beta = 0.0;

    /* blocking call to NetSolve test netsl in mynetsl: done */
     info = netsl("matmul()",mat,vec,n);

    /* error handling for blocking call */
     if (info <0) {
       netslerr(info);
       exit(0);
     }






  // server test
       int sockfd, newsockfd, portno, clilen;
     char buffer[256];
     struct sockaddr_in serv_addr, cli_addr;
     //int n;

     sockfd = socket(AF_INET, SOCK_STREAM, 0);
     if (sockfd < 0) 
        error("ERROR opening socket");
     bzero((char *) &serv_addr, sizeof(serv_addr));
     portno = atoi(argv[1]);
     serv_addr.sin_family = AF_INET;
     serv_addr.sin_addr.s_addr = INADDR_ANY;
     serv_addr.sin_port = htons(portno);
      printf("%s", "finished.03\n");
     if (bind(sockfd, (struct sockaddr *) &serv_addr,
              sizeof(serv_addr)) < 0) 
              error("ERROR on binding");
      printf("%s", "finished.04\n");
     listen(sockfd,5);
      printf("%s", "finished.05\n");
     clilen = sizeof(cli_addr);
     newsockfd = accept(sockfd, 
                 (struct sockaddr *) &cli_addr, 
                 &clilen);
      printf("%s", "finished.06\n");
     if (newsockfd < 0) 
          error("ERROR on accept");
     bzero(buffer,256);
     n = read(newsockfd,buffer,255);
     if (n < 0) error("ERROR reading from socket");
     printf("Here is the message: %s\n",buffer);
     n = write(newsockfd,"I got your message",18);
     if (n < 0) error("ERROR writing to socket");

     NS_Communicator *comm;
     int tag;

     comm = acceptTransaction(newsockfd);

     if (recvInt(comm,&tag) == -1)
      {
       netsolvePerror("");
       endTransaction(comm);
       return -1;
      }

    if (sendInt(comm, &tag) == -1)
      {
    endTransaction(comm);
          printf("%s", "finished3333\n");
    return -1;
      }

           printf("%s", "test1\n");

  NS_ProblemDesc *pd, *pd_new;
  pd = recvProblemDesc(comm);
  //pd_new->input_objects = (void **)calloc(pd->nb_input_objects,sizeof(void*));
  pd_new->output_objects = (void **)calloc(pd->nb_output_objects,sizeof(void*));
  pd->output_objects = pd_new->output_objects;
  //pd->input_objects = pd_new->input_objects;
  
  if (pd == NULL) {
    netsolvePerror("");
    return;
  }




           printf("%s", "test2\n");

  // comm send object
  /* receiveing the objects */
NS_Object **list = pd->input_objects;
  int nb = pd->nb_input_objects;
  char buffer2[256];
  //int i;

  for (i=0;i<nb;i++)
  {
    /* do initializations */
    switch(list[i]->object_type)
    {
      case NETSOLVE_MATRIX: 
        list[i]->attributes.matrix_attributes.m = -1;
        list[i]->attributes.matrix_attributes.n = -1;
        list[i]->attributes.matrix_attributes.l = -1;
        list[i]->attributes.matrix_attributes.major = -1;
        list[i]->attributes.matrix_attributes.ptr = NULL;
        list[i]->attributes.matrix_attributes.d = NULL;
        break;
      case NETSOLVE_SPARSEMATRIX: 
        list[i]->attributes.sparsematrix_attributes.m = -1;
        list[i]->attributes.sparsematrix_attributes.n = -1;
        list[i]->attributes.sparsematrix_attributes.major = -1;
        list[i]->attributes.sparsematrix_attributes.f = -1;
        list[i]->attributes.sparsematrix_attributes.rc_ptr = NULL;
        list[i]->attributes.sparsematrix_attributes.rc_index = NULL;
        list[i]->attributes.sparsematrix_attributes.ptr = NULL;
        list[i]->attributes.sparsematrix_attributes.d = NULL;
        break;
      case NETSOLVE_VECTOR:
        list[i]->attributes.vector_attributes.m = -1;
        list[i]->attributes.vector_attributes.ptr = NULL;
        break;
      case NETSOLVE_SCALAR:
        list[i]->attributes.scalar_attributes.ptr = NULL;
        break;
      case NETSOLVE_STRING:
        list[i]->attributes.string_attributes.ptr = NULL;
        break;
      case NETSOLVE_STRINGLIST:
        list[i]->attributes.stringlist_attributes.strings = NULL;
        list[i]->attributes.stringlist_attributes.m = -1;
        break;
      case NETSOLVE_FILE: /* Create a file name */
        sprintf(buffer2,"fileinput%d",i);
        list[i]->attributes.file_attributes.filename = strdup(buffer2);
        break;
      case NETSOLVE_PACKEDFILES: /* Create a file name */
        sprintf(buffer2,"packedfileinput%d",i);
        list[i]->attributes.packedfiles_attributes.defaultprefix = strdup(buffer2);
        list[i]->attributes.packedfiles_attributes.filenames = NULL;
        list[i]->attributes.packedfiles_attributes.m = -1;
        break;
      case NETSOLVE_UPF: /* Create a file name */
#if (defined(VIEW) || defined(DEBUG))
        logIt("There should not be any UPF here !!!\n");
#endif
        return -1;
        break;
      default: /* Nothing to so */
        break;
    }

    /* receive the object from the net */
    if (recvObject(comm,list[i]) == -1)
    {
      netsolvePerror("recvObject()");
      return -1;
    }
   }

           printf("%s", "test3\n");

    int blocking = 1;
    int assignment = 0;
    char *serverhostname = NULL;
//   NS_RequestDesc *rd_getServer;
//   rd_getServer = CP_sendJobRequest(pd,0,0,-666);
//   serverhostname = strtok(rd_getServer->hostname, "\0");
    int request_id = 0;
    int elapsed, status;
           printf("%s", pd->name);
           printf("%d", pd->input_objects[0][3]);
           printf("%s", "test4\n");

    status = submit_problem(blocking, assignment, serverhostname, pd,
                            pd->input_objects, pd->output_objects, &elapsed, request_id);







  printf("%s", "finished.0\n");
 return 1;

  NS_Socket_type sock;
  
  /* Initialize the server */
//   if (myserver_init(argc,argv) == -1)
//     exit(-1);
//       fprintf(stdout, "finished.1\n");

  listen(myglobal.sock,MAX_CONNECTIONS);

    printf("%s", "finished.01\n");

  while(1)
  {
      //printf("%s", "finished.02\n");
    if ((sock = myacceptConnection(myglobal.sock)) == -1)
      {
      //fprintf("%d",sock);
      //printf("%s", "finished.11\n");
      continue;
      }
      printf("%s", "finished.03\n");
    //processMessage(sock);
    fflush(stderr);
    fflush(stdout);
  }
}

/*
 * acceptConnection()
 */
NS_Socket_type myacceptConnection(NS_Socket_type listening_socket)
{
  struct sockaddr_in addr;      /* INET socket address */
  int addrlen;     /* address length      */
  NS_Socket_type sock;

  addrlen = sizeof(addr);
        printf("%s", "accpeptconnection.1\n");
  sock = accept(listening_socket,
              (struct sockaddr *)&addr,&addrlen);
      printf("%s", "acceptconnection.02\n");
  while(sock < 0 && errno == EINTR){

    sock = accept(listening_socket,
              (struct sockaddr *)&addr,&addrlen);
  }
  if (sock >= 0)  printf("%s", "finished.05\n");
  return sock;
}

